==> api_server_controller-0.1.6.1a0-py3-none-any.whl: asc/core/tools/delivery/melhorenvio/control.py <==
import logging
from sys import argv
from typing import List
from datetime import datetime

from sqlalchemy import update

from asc.core.ABC.Control import ControlTool
from asc.core.tools.delivery.melhorenvio.api import MelhorEnvio
from asc.core.tools.delivery.melhorenvio.data import OrderSearch
from asc.models.melhorenvio import Order
from asc.models.melhorenvio import OrderDelivery
from asc.models.melhorenvio import MelhorenvioState
from asc.models.melhorenvio import OrderDeliveryStatus

logging.basicConfig(level=logging.INFO)
"""
    Para utilização/renovação de token melhor envio:
        1 - obter o URL e enviar para o cliente
        2 - postar oauth2_code para registrar autorização
        3 - aplicar refresh token para atualizar oauth2_code
"""
ID_ORDER_DELIVERY = 187


class ControlToolHandler(ControlTool):

    ME:MelhorEnvio
    ME_DATA__state:MelhorenvioState
    ME_DATA__status:List[OrderDeliveryStatus]

    def __init__(self):
        super(ControlToolHandler, self).__init__()

        print('SYS-ARGV',argv)

        # Defines data to control
        self.ME_DATA__state = self.session.query(MelhorenvioState).first()
        self.ME_DATA__status= self.session.query(OrderDeliveryStatus).all()

        # Defines auth_code into API
        self.ME = MelhorEnvio(code=self.ME_DATA__state.oauth2_code)

    def find_status_code_from_str(self,name:str):
        """ Retorna código do status. Registra quando não existe.
        :param name: nome do status recebido.
        :return: int
        """

        # Encontra código em cache
        for status in self.ME_DATA__status:
            print('STATUS',status)
            print('STATUS-NAME',status.name)
            s = status.as_dict()
            print('STATUS-DICT',s)

            if status.name == name:
                return status.id_order_delivery_status

        new_status = OrderDeliveryStatus()
        new_status.name = name

        # Registra novo status e armazena em cache.
        self.ME_DATA__status.append(new_status)
        self.session.add(new_status)
        self.session.flush()

        return new_status.id_order_delivery_status

    def generate_auth_url(self):
        """ Obter a url de autorização. Este url deve ser enviada
            para o cliente conceder autorização. O callback registra
            os códigos de autorização automaticamente.
        :return: link de autorização para ser enviado ao cliente.
        """
        return self.ME.GET_oauth_code()

    def register_token(self):
        """
        Registrar token na melhor envio e autoriza para utilização
        :return: dict
        """

        # Post oauth token.
        response:dict = self.ME.POST_oauth_token()

        return response

    def refresh_token(self):
        """ Atualiza bearer token para melhorenvio """

        # Refresh oauth token.
        response:dict = self.ME.POST_oauth_token_refresh(self.ME_DATA__state.code_refresh)

        response_access_token = response.get('access_token')
        response_refresh_token= response.get('refresh_token')

        self.ME_DATA__state.code_refresh = response_refresh_token
        self.ME_DATA__state.oauth2_code = response_access_token
        self.ME_DATA__state.dt_update = datetime.now()
        self.commit()

        self.ME.set_oauth_code(self.ME_DATA__state.oauth2_code)

        return response

    def refresh_not_arrived(self):
        """ Atualiza informações das entregas que ainda não chegaram.
        :return:
        """

        query_order_delivery= self.session\
            .query(
                OrderDelivery.id_order_delivery.label('id'),
                OrderDelivery.is_errored,
                OrderDelivery.dt_arrived,
                OrderDelivery.cd_tracking,
                OrderDelivery.dt_canceled,
                OrderDelivery.id_melhorenvio_cart,
                OrderDelivery.id_order_delivery_status)\
            .join(Order,
                  Order.id_order_delivery==OrderDelivery.id_order_delivery)\
            .filter(
                OrderDelivery.is_errored==False,
                OrderDelivery.dt_arrived.is_(None),
                #OrderDelivery.id_order_delivery_status<=4,
                Order.id_shipment>1
            #    OrderDelivery.id_order_delivery==ID_ORDER_DELIVERY
            ).all()

        for delivery in query_order_delivery:
            print("------> DELIVERY_DATABASE",delivery)

            search_by = delivery.id_melhorenvio_cart or delivery.cd_tracking

            if not search_by:
                # Se não há por onde buscar, não executa a busca.
                continue

            # Obtém os dados atualizados da entrega na melhor envio.
            delivery_response = self.ME\
                .GET_order_search(q=search_by)
            print("------> DELIVERY_RESPONSE",delivery_response)

            delivery_as_dict = delivery._asdict()

            if 'error' in delivery_response:
                stmt = update(OrderDelivery)\
                    .where(OrderDelivery.id_order_delivery
                           == delivery.id)\
                    .values(is_errored=True).\
                    execution_options(synchronize_session="fetch")
                self.session.execute(stmt)
                continue

            # Obtém dados da resposta existente
            delivery_response = OrderSearch(**delivery_response[0])

            # Registra ou atualiza status da entrega.
            status = self.find_status_code_from_str(delivery_response.status)
            delivery_as_dict['id_order_delivery_status'] = status

            # Registra data de cancelamento.
            if delivery_response.canceled_at:
                delivery_as_dict['dt_canceled'] = delivery_response.canceled_at

            # Registra data de chegada.
            if delivery_response.delivered_at:
                delivery_as_dict['dt_arrived'] = delivery_response.delivered_at

                stmt = update(Order)\
                    .where(
                    #    OrderDelivery.id_order_delivery == Order.id_order_delivery,
                        Order.id_order_delivery == delivery.id)\
                    .values(
                        id_order_status=5
                    ). \
                    execution_options(synchronize_session="fetch")

                self.session.execute(stmt)

            # Atualiza código de rastreio quando necessário.
            if delivery.cd_tracking != delivery_response.tracking:
                delivery_as_dict['cd_tracking'] = delivery_response.tracking

            print(delivery_as_dict)

            stmt = update(OrderDelivery) \
                .where(OrderDelivery.id_order_delivery
                       == delivery.id) \
                .values(
                    id_order_delivery_status=status,
                    dt_canceled=delivery_as_dict['dt_canceled'],
                    cd_tracking=delivery_as_dict['cd_tracking'],
                    dt_arrived=delivery_as_dict['dt_arrived']
            ). \
                execution_options(synchronize_session="fetch")

            self.session.execute(stmt)

        self.commit()
        return query_order_delivery

==> api_server_controller-0.2.2a5-py3-none-any.whl: asc/core/tools/delivery/melhorenvio/control.py <==
import logging
from sys import argv
from typing import List
from datetime import datetime

from sqlalchemy import update

from asc.core.ABC.Control import ControlTool
from asc.core.tools.delivery.melhorenvio.api import MelhorEnvio
from asc.core.tools.delivery.melhorenvio.data import OrderSearch
from asc.models.marketplace import Order
from asc.models.melhorenvio import OrderDelivery
from asc.models.melhorenvio import MelhorenvioState
from asc.models.melhorenvio import OrderDeliveryStatus

logging.basicConfig(level=logging.INFO)
"""
    Para utilização/renovação de token melhor envio:
        1 - obter o URL e enviar para o cliente
        2 - postar oauth2_code para registrar autorização
        3 - aplicar refresh token para atualizar oauth2_code
"""
ID_ORDER_DELIVERY = 187


class ControlToolHandler(ControlTool):

    ME:MelhorEnvio
    ME_DATA__state:MelhorenvioState
    ME_DATA__status:List[OrderDeliveryStatus]

    def __init__(self):
        super(ControlToolHandler, self).__init__()

        print('SYS-ARGV',argv)

        # Defines data to control
        self.ME_DATA__state = self.session.query(MelhorenvioState).first()
        self.ME_DATA__status= self.session.query(OrderDeliveryStatus).all()

        # Defines auth_code into API
        self.ME = MelhorEnvio(code=self.ME_DATA__state.oauth2_code)

    def find_status_code_from_str(self,name:str):
        """ Retorna código do status. Registra quando não existe.
        :param name: nome do status recebido.
        :return: int
        """

        # Encontra código em cache
        for status in self.ME_DATA__status:
            print('STATUS',status)
            print('STATUS-NAME',status.name)
            s = status.as_dict()
            print('STATUS-DICT',s)

            if status.name == name:
                return status.id_order_delivery_status

        new_status = OrderDeliveryStatus()
        new_status.name = name

        # Registra novo status e armazena em cache.
        self.ME_DATA__status.append(new_status)
        self.session.add(new_status)
        self.session.flush()

        return new_status.id_order_delivery_status

    def generate_auth_url(self):
        """ Obter a url de autorização. Este url deve ser enviada
            para o cliente conceder autorização. O callback registra
            os códigos de autorização automaticamente.
        :return: link de autorização para ser enviado ao cliente.
        """
        return self.ME.GET_oauth_code()

    def register_token(self):
        """
        Registrar token na melhor envio e autoriza para utilização
        :return: dict
        """

        # Post oauth token.
        response:dict = self.ME.POST_oauth_token()

        return response

    def refresh_token(self):
        """ Atualiza bearer token para melhorenvio """

        # Refresh oauth token.
        response:dict = self.ME.POST_oauth_token_refresh(self.ME_DATA__state.code_refresh)

        response_access_token = response.get('access_token')
        response_refresh_token= response.get('refresh_token')

        self.ME_DATA__state.code_refresh = response_refresh_token
        self.ME_DATA__state.oauth2_code = response_access_token
        self.ME_DATA__state.dt_update = datetime.now()
        self.commit()

        self.ME.set_oauth_code(self.ME_DATA__state.oauth2_code)

        return response

    def refresh_not_arrived(self):
        """ Atualiza informações das entregas que ainda não chegaram.
        :return:
        """

        query_order_delivery= self.session\
            .query(
                OrderDelivery.id_order_delivery.label('id'),
                OrderDelivery.is_errored,
                OrderDelivery.dt_arrived,
                OrderDelivery.cd_tracking,
                OrderDelivery.dt_canceled,
                OrderDelivery.id_melhorenvio_cart,
                OrderDelivery.id_order_delivery_status)\
            .join(Order,
                  Order.id_order_delivery==OrderDelivery.id_order_delivery)\
            .filter(
                OrderDelivery.is_errored==False,
                OrderDelivery.dt_arrived.is_(None),
                #OrderDelivery.id_order_delivery_status<=4,
                Order.id_shipment>1
            #    OrderDelivery.id_order_delivery==ID_ORDER_DELIVERY
            ).all()

        for delivery in query_order_delivery:
            print("------> DELIVERY_DATABASE",delivery)

            search_by = delivery.id_melhorenvio_cart or delivery.cd_tracking

            if not search_by:
                # Se não há por onde buscar, não executa a busca.
                continue

            # Obtém os dados atualizados da entrega na melhor envio.
            delivery_response = self.ME\
                .GET_order_search(q=search_by)
            print("------> DELIVERY_RESPONSE",delivery_response)

            delivery_as_dict = delivery.asdict()

            if 'error' in delivery_response:
                stmt = update(OrderDelivery)\
                    .where(OrderDelivery.id_order_delivery
                           == delivery.id)\
                    .values(is_errored=True).\
                    execution_options(synchronize_session="fetch")
                self.session.execute(stmt)
                continue

            # Obtém dados da resposta existente
            delivery_response = OrderSearch(**delivery_response[0])

            # Registra ou atualiza status da entrega.
            status = self.find_status_code_from_str(delivery_response.status)
            delivery_as_dict['id_order_delivery_status'] = status

            # Registra data de cancelamento.
            if delivery_response.canceled_at:
                delivery_as_dict['dt_canceled'] = delivery_response.canceled_at

            # Registra data de chegada.
            if delivery_response.delivered_at:
                delivery_as_dict['dt_arrived'] = delivery_response.delivered_at

                stmt = update(Order)\
                    .where(
                    #    OrderDelivery.id_order_delivery == Order.id_order_delivery,
                        Order.id_order_delivery == delivery.id)\
                    .values(
                        id_order_status=5
                    ). \
                    execution_options(synchronize_session="fetch")

                self.session.execute(stmt)

            # Atualiza código de rastreio quando necessário.
            if delivery.cd_tracking != delivery_response.tracking:
                delivery_as_dict['cd_tracking'] = delivery_response.tracking

            print(delivery_as_dict)

            stmt = update(OrderDelivery) \
                .where(OrderDelivery.id_order_delivery
                       == delivery.id) \
                .values(
                    id_order_delivery_status=status,
                    dt_canceled=delivery_as_dict['dt_canceled'],
                    cd_tracking=delivery_as_dict['cd_tracking'],
                    dt_arrived=delivery_as_dict['dt_arrived']
            ). \
                execution_options(synchronize_session="fetch")

            self.session.execute(stmt)

        self.commit()
        return query_order_delivery
